use iced::Subscription;
use iced::Clipboard;
use iced::Command;
use iced::Application;
use iced::Container;
use iced::Row;
use iced::Element;
use iced::Length;
use iced::Align;
use iced::executor;
use iced_native::subscription;
use iced_native::keyboard;
use iced_native::event::Event;

mod lyrics;
mod styles;

use styles::Theme;

#[derive(Clone, Debug)]
pub struct DelyriumApp {
	lyrics_component: lyrics::Lyrics,
	theme: Theme,
}

#[derive(Clone, Debug)]
pub enum Message {
	LyricChanged {
		line_no: usize,
		new_value: String,
	},
	LineAdvanced(usize),
	PasteSent,
}

impl Application for DelyriumApp {
	type Message = Message;
	type Executor = executor::Default;
	type Flags = ();

	fn new(_: Self::Flags) -> (Self, Command<Message>) {
		(
			DelyriumApp {
				lyrics_component: lyrics::Lyrics::new(),
				theme: Theme::default(),
			},
			Command::none(),
		)
	}

	fn title(&self) -> String {
		String::from("Delyrium")
	}

	fn update(&mut self, message: Message, clipboard: &mut Clipboard) -> Command<Message>{
		match message {
			Message::LyricChanged { line_no, new_value } => {
				self.lyrics_component.update_line(new_value, line_no);
			},
			Message::LineAdvanced(current_line) => {
				self.lyrics_component.advance_line(current_line);
			},
			Message::PasteSent => {
				let clip_text = clipboard.read().unwrap_or(String::new());
				let clip_pasted_len = clip_text.chars()
					.filter(|c| *c != '\r' && *c != '\n')
					.count();
				let line = self.lyrics_component.current_line_mut().1;
				line.value.truncate(line.value.len() - clip_pasted_len);
				self.lyrics_component.insert_text(clip_text);
			}
		}
		Command::none()
	}

	fn view(&mut self) -> Element<Message> {
		Container::new(
			Row::new()
				.push(self.lyrics_component.view(self.theme))
		)
		.align_y(Align::Center)
		.style(self.theme)
		.height(Length::Fill)
		.into()
	}

	fn subscription(&self) -> Subscription<Message> {
		subscription::events_with(|event, _| {
			match event {
				Event::Keyboard(keyboard::Event::KeyPressed {key_code, modifiers}) => {
					match (key_code, modifiers) {
						(
							keyboard::KeyCode::V,
							keyboard::Modifiers { control, .. }
						) if control == true => {
							Some(Message::PasteSent)
						}
						_ => { None }
					}
				}
				_ => { None }
			}
		})
	}
}
